@@ -48,9 +48,6 @@ as that of the covered work.  */
 #include "c-strcasestr.h"
 #include "xstrndup.h"
 
-/* RFC3987 section 3.1 mandates STD3 ASCII RULES */
-#define IDNA_FLAGS  IDNA_USE_STD3_ASCII_RULES
-
 /* Note: locale encoding is kept in options struct (opt.locale) */
 
 /* Given a string containing "charset=XXX", return the encoding if found,
@@ -167,7 +164,7 @@ do_conversion (const char *tocode, const char *fromcode, char const *in_org, siz
             if (!strchr(in_org, '@') && !strchr(*out, '@'))
               debug_logprintf ("converted '%s' (%s) -> '%s' (%s)\n", in_org, fromcode, *out, tocode);
             else
-              debug_logprintf ("%s: logging suppressed, strings may contain password\n", __func__);
+              debug_logprintf ("logging suppressed, strings may contain password\n");
           }
           return true;
         }
@@ -209,7 +206,7 @@ do_conversion (const char *tocode, const char *fromcode, char const *in_org, siz
       if (!strchr(in_org, '@') && !strchr(*out, '@'))
         debug_logprintf ("converted '%s' (%s) -> '%s' (%s)\n", in_org, fromcode, *out, tocode);
       else
-        debug_logprintf ("%s: logging suppressed, strings may contain password\n", __func__);
+        debug_logprintf ("logging suppressed, strings may contain password\n");
     }
     return false;
 }
@@ -289,6 +286,8 @@ idn_encode (const struct iri *i, const char *host)
   xfree (lower);
 #endif
 
+  xfree (utf8_encoded);
+
   return ret == IDN2_OK ? ascii_encoded : NULL;
 }
 
