@@ -75,7 +75,7 @@ search_netrc (const char *host, const char **acc, const char **passwd,
 #ifdef __VMS
 
       int err;
-      struct_stat buf;
+      struct stat buf;
       char *path = "SYS$LOGIN:.netrc";
 
       netrc_list = NULL;
@@ -94,7 +94,7 @@ search_netrc (const char *host, const char **acc, const char **passwd,
       if (home)
         {
           int err;
-          struct_stat buf;
+          struct stat buf;
           char *path = (char *)alloca (strlen (home) + 1
                                        + strlen (NETRC_FILE_NAME) + 1);
           sprintf (path, "%s/%s", home, NETRC_FILE_NAME);
@@ -439,7 +439,7 @@ free_netrc(acc_t *l)
 int
 main (int argc, char **argv)
 {
-  struct_stat sb;
+  struct stat sb;
   char *program_name, *file, *target;
   acc_t *head, *a;
 
